////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  awbrsubmodule.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "awbrsubmodule.h"
#include "parametersetmanager.h"

AWBRSUBMODULE_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool config2AOutputCheckAwbType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbGains") == 0)
    {
        __result = config2AOutputCheckAwbGains.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbCCT") == 0)
    {
        __result = config2AOutputCheckAwbCCT.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool config2AOutputCheckAwbType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbGains") == 0)
    {
        __result = config2AOutputCheckAwbGains.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbCCT") == 0)
    {
        __result = config2AOutputCheckAwbCCT.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool config2AOutputCheckAwbType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // config2AOutputCheckAwbGains
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("config2AOutputCheckAwbGains", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbGains") == 0)
    {
        __result = config2AOutputCheckAwbGains.GetInfo(&__path[__pos], __info);
    }

    // config2AOutputCheckAwbCCT
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("config2AOutputCheckAwbCCT", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwbCCT") == 0)
    {
        __result = config2AOutputCheckAwbCCT.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbGainsStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "r_threshold_start") == 0)
    {
        ParameterUtils::DoubleToString((double)r_threshold_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "r_threshold_end") == 0)
    {
        ParameterUtils::DoubleToString((double)r_threshold_end, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_start") == 0)
    {
        ParameterUtils::DoubleToString((double)g_threshold_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_end") == 0)
    {
        ParameterUtils::DoubleToString((double)g_threshold_end, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_start") == 0)
    {
        ParameterUtils::DoubleToString((double)b_threshold_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_end") == 0)
    {
        ParameterUtils::DoubleToString((double)b_threshold_end, __value, __size);
        __result = true;
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbGainsStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "r_threshold_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            r_threshold_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "r_threshold_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            r_threshold_end = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            g_threshold_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            g_threshold_end = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            b_threshold_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            b_threshold_end = (float)dblValue; 
        }
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbGainsStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // r_threshold_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("r_threshold_start", ParameterParserDataType::Float, (double)r_threshold_start));
    }
    else if (PARAMETER_STRCMP(__name, "r_threshold_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)r_threshold_start));
    }

    // r_threshold_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("r_threshold_end", ParameterParserDataType::Float, (double)r_threshold_end));
    }
    else if (PARAMETER_STRCMP(__name, "r_threshold_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)r_threshold_end));
    }

    // g_threshold_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("g_threshold_start", ParameterParserDataType::Float, (double)g_threshold_start));
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)g_threshold_start));
    }

    // g_threshold_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("g_threshold_end", ParameterParserDataType::Float, (double)g_threshold_end));
    }
    else if (PARAMETER_STRCMP(__name, "g_threshold_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)g_threshold_end));
    }

    // b_threshold_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("b_threshold_start", ParameterParserDataType::Float, (double)b_threshold_start));
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)b_threshold_start));
    }

    // b_threshold_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("b_threshold_end", ParameterParserDataType::Float, (double)b_threshold_end));
    }
    else if (PARAMETER_STRCMP(__name, "b_threshold_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)b_threshold_end));
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbCCTStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "cct_threshold_start") == 0)
    {
        ParameterUtils::DoubleToString((double)cct_threshold_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "cct_threshold_end") == 0)
    {
        ParameterUtils::DoubleToString((double)cct_threshold_end, __value, __size);
        __result = true;
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbCCTStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "cct_threshold_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            cct_threshold_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "cct_threshold_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            cct_threshold_end = (float)dblValue; 
        }
    }

    return __result;
}

bool config2AOutputCheckAwbType::config2AOutputCheckAwbCCTStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // cct_threshold_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cct_threshold_start", ParameterParserDataType::Float, (double)cct_threshold_start));
    }
    else if (PARAMETER_STRCMP(__name, "cct_threshold_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)cct_threshold_start));
    }

    // cct_threshold_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cct_threshold_end", ParameterParserDataType::Float, (double)cct_threshold_end));
    }
    else if (PARAMETER_STRCMP(__name, "cct_threshold_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)cct_threshold_end));
    }

    return __result;
}

bool config2AInputCheckAwbType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "config2AInputCheckValidStatsRatio") == 0)
    {
        ParameterUtils::DoubleToString((double)config2AInputCheckValidStatsRatio, __value, __size);
        __result = true;
    }

    return __result;
}

bool config2AInputCheckAwbType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "config2AInputCheckValidStatsRatio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            config2AInputCheckValidStatsRatio = (float)dblValue; 
        }
    }

    return __result;
}

bool config2AInputCheckAwbType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // config2AInputCheckValidStatsRatio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("config2AInputCheckValidStatsRatio", ParameterParserDataType::Float, (double)config2AInputCheckValidStatsRatio));
    }
    else if (PARAMETER_STRCMP(__name, "config2AInputCheckValidStatsRatio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)config2AInputCheckValidStatsRatio));
    }

    return __result;
}

bool cctGainAdjType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "adjFactors") == 0)
    {
        __result = adjFactors.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool cctGainAdjType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "adjFactors") == 0)
    {
        __result = adjFactors.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool cctGainAdjType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // adjFactors
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("adjFactors", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "adjFactors") == 0)
    {
        __result = adjFactors.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool tmpConvType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "camStartConvSpeed") == 0)
    {
        ParameterUtils::DoubleToString((double)camStartConvSpeed, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "camStartFrameCnt") == 0)
    {
        ParameterUtils::DoubleToString((double)camStartFrameCnt, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "normalConvSpeed") == 0)
    {
        ParameterUtils::DoubleToString((double)normalConvSpeed, __value, __size);
        __result = true;
    }

    return __result;
}

bool tmpConvType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "camStartConvSpeed") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            camStartConvSpeed = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "camStartFrameCnt") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            camStartFrameCnt = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "normalConvSpeed") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            normalConvSpeed = (float)dblValue; 
        }
    }

    return __result;
}

bool tmpConvType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // camStartConvSpeed
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("camStartConvSpeed", ParameterParserDataType::Float, (double)camStartConvSpeed));
    }
    else if (PARAMETER_STRCMP(__name, "camStartConvSpeed") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)camStartConvSpeed));
    }

    // camStartFrameCnt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("camStartFrameCnt", ParameterParserDataType::Int32, (double)camStartFrameCnt, 0));
    }
    else if (PARAMETER_STRCMP(__name, "camStartFrameCnt") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)camStartFrameCnt, 0));
    }

    // normalConvSpeed
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("normalConvSpeed", ParameterParserDataType::Float, (double)normalConvSpeed));
    }
    else if (PARAMETER_STRCMP(__name, "normalConvSpeed") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)normalConvSpeed));
    }

    return __result;
}

bool dfsType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dfsTrigger") == 0)
    {
        if (__index < dfsTriggerCount)
        {
            __result = dfsTrigger[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool dfsType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dfsTrigger") == 0)
    {
        if (__index < dfsTriggerCount)
        {
            __result = dfsTrigger[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool dfsType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // dfsTrigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dfsTrigger", ParameterParserDataType::Structure, true, false, dfsTriggerCount));
    }
    else if (PARAMETER_STRCMP(__name, "dfsTrigger") == 0)
    {
        if (__index < dfsTriggerCount)
        {
            __result = dfsTrigger[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, dfsTriggerCount));
        }
    }

    return __result;
}

bool sensorCalibrationType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "numOfCalFactors") == 0)
    {
        ParameterUtils::DoubleToString((double)numOfCalFactors, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "cctOfCalFactors") == 0)
    {
        if (__index < 6)
        {
            ParameterUtils::DoubleToString((double)cctOfCalFactors[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 6; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)cctOfCalFactors[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "goldenModuleCal") == 0)
    {
        if (__index < 15)
        {
            __result = goldenModuleCal[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool sensorCalibrationType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "numOfCalFactors") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            numOfCalFactors = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "cctOfCalFactors") == 0)
    {
        if (__index < 6)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                cctOfCalFactors[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 6)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    cctOfCalFactors[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "goldenModuleCal") == 0)
    {
        if (__index < 15)
        {
            __result = goldenModuleCal[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool sensorCalibrationType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // numOfCalFactors
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("numOfCalFactors", ParameterParserDataType::Int32, (double)numOfCalFactors, 0));
    }
    else if (PARAMETER_STRCMP(__name, "numOfCalFactors") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)numOfCalFactors, 0));
    }

    // cctOfCalFactors
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cctOfCalFactors", ParameterParserDataType::Float, true, false, 6));
    }

    // goldenModuleCal
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("goldenModuleCal", ParameterParserDataType::Structure, true, false, 15));
    }
    else if (PARAMETER_STRCMP(__name, "goldenModuleCal") == 0)
    {
        if (__index < 15)
        {
            __result = goldenModuleCal[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, 15));
        }
    }

    return __result;
}

bool blackleveloffsetType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "OffsetSubtraction") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)OffsetSubtraction[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)OffsetSubtraction[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool blackleveloffsetType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "OffsetSubtraction") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                OffsetSubtraction[__index] = (INT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    OffsetSubtraction[__index] = (INT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool blackleveloffsetType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // OffsetSubtraction
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("OffsetSubtraction", ParameterParserDataType::Int32, true, false, 3));
    }

    return __result;
}

bool inverseWBConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "InverseWBGain") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)InverseWBGain[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)InverseWBGain[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool inverseWBConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "InverseWBGain") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                InverseWBGain[__index] = (INT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    InverseWBGain[__index] = (INT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool inverseWBConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // InverseWBGain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("InverseWBGain", ParameterParserDataType::Int32, true, false, 3));
    }

    return __result;
}

bool bitDepthConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "bitDepth") == 0)
    {
        ParameterUtils::DoubleToString((double)bitDepth, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool bitDepthConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "bitDepth") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            bitDepth = (INT32)dblValue; 
        }
    }

    return __result;
}

bool bitDepthConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // bitDepth
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bitDepth", ParameterParserDataType::Int32, (double)bitDepth, 0));
    }
    else if (PARAMETER_STRCMP(__name, "bitDepth") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)bitDepth, 0));
    }

    return __result;
}

bool BGStatsConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "numOfRegionsHorizontal") == 0)
    {
        ParameterUtils::DoubleToString((double)numOfRegionsHorizontal, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "numOfRegionsVertical") == 0)
    {
        ParameterUtils::DoubleToString((double)numOfRegionsVertical, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "enableStatsQuadSync") == 0)
    {
        ParameterUtils::DoubleToString((double)enableStatsQuadSync, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "enableSaturatedStats") == 0)
    {
        ParameterUtils::DoubleToString((double)enableSaturatedStats, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "enableYComputedStats") == 0)
    {
        ParameterUtils::DoubleToString((double)enableYComputedStats, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "YComputationColorWeight") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)YComputationColorWeight[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)YComputationColorWeight[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "minStatsChannelThreshold") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)minStatsChannelThreshold[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)minStatsChannelThreshold[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "maxStatsChannelThreshold") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)maxStatsChannelThreshold[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)maxStatsChannelThreshold[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "HWStatsSkipFactor") == 0)
    {
        ParameterUtils::DoubleToString((double)HWStatsSkipFactor, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ROIConfigControl") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)ROIConfigControl, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "bitDepthConfig") == 0)
    {
        __result = bitDepthConfig.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "InverseWBConfig") == 0)
    {
        __result = InverseWBConfig.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "BlackLevelOffsetConfig") == 0)
    {
        __result = BlackLevelOffsetConfig.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool BGStatsConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "numOfRegionsHorizontal") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            numOfRegionsHorizontal = (UINT16)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "numOfRegionsVertical") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            numOfRegionsVertical = (UINT16)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableStatsQuadSync") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableStatsQuadSync = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableSaturatedStats") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableSaturatedStats = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableYComputedStats") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableYComputedStats = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "YComputationColorWeight") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                YComputationColorWeight[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    YComputationColorWeight[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "minStatsChannelThreshold") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                minStatsChannelThreshold[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    minStatsChannelThreshold[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "maxStatsChannelThreshold") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                maxStatsChannelThreshold[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    maxStatsChannelThreshold[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "HWStatsSkipFactor") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            HWStatsSkipFactor = (UINT8)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ROIConfigControl") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ROIConfigControl = (BGStatsROIConfigType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "bitDepthConfig") == 0)
    {
        __result = bitDepthConfig.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "InverseWBConfig") == 0)
    {
        __result = InverseWBConfig.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "BlackLevelOffsetConfig") == 0)
    {
        __result = BlackLevelOffsetConfig.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool BGStatsConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // numOfRegionsHorizontal
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("numOfRegionsHorizontal", ParameterParserDataType::UInt16, (double)numOfRegionsHorizontal, 0));
    }
    else if (PARAMETER_STRCMP(__name, "numOfRegionsHorizontal") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt16, (double)numOfRegionsHorizontal, 0));
    }

    // numOfRegionsVertical
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("numOfRegionsVertical", ParameterParserDataType::UInt16, (double)numOfRegionsVertical, 0));
    }
    else if (PARAMETER_STRCMP(__name, "numOfRegionsVertical") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt16, (double)numOfRegionsVertical, 0));
    }

    // enableStatsQuadSync
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableStatsQuadSync", ParameterParserDataType::Boolean, (double)enableStatsQuadSync, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableStatsQuadSync") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableStatsQuadSync, 0));
    }

    // enableSaturatedStats
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableSaturatedStats", ParameterParserDataType::Boolean, (double)enableSaturatedStats, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableSaturatedStats") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableSaturatedStats, 0));
    }

    // enableYComputedStats
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableYComputedStats", ParameterParserDataType::Boolean, (double)enableYComputedStats, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableYComputedStats") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableYComputedStats, 0));
    }

    // YComputationColorWeight
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("YComputationColorWeight", ParameterParserDataType::Float, true, false, 3));
    }

    // minStatsChannelThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minStatsChannelThreshold", ParameterParserDataType::Float, true, false, 4));
    }

    // maxStatsChannelThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("maxStatsChannelThreshold", ParameterParserDataType::Float, true, false, 4));
    }

    // HWStatsSkipFactor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("HWStatsSkipFactor", ParameterParserDataType::UInt8, (double)HWStatsSkipFactor, 0));
    }
    else if (PARAMETER_STRCMP(__name, "HWStatsSkipFactor") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt8, (double)HWStatsSkipFactor, 0));
    }

    // ROIConfigControl
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ROIConfigControl", ParameterParserDataType::Enum, (double)(UINT32)ROIConfigControl, 0));
    }
    else if (PARAMETER_STRCMP(__name, "ROIConfigControl") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)ROIConfigControl, 0));
    }

    // bitDepthConfig
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bitDepthConfig", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "bitDepthConfig") == 0)
    {
        __result = bitDepthConfig.GetInfo(&__path[__pos], __info);
    }

    // InverseWBConfig
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("InverseWBConfig", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "InverseWBConfig") == 0)
    {
        __result = InverseWBConfig.GetInfo(&__path[__pos], __info);
    }

    // BlackLevelOffsetConfig
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("BlackLevelOffsetConfig", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "BlackLevelOffsetConfig") == 0)
    {
        __result = BlackLevelOffsetConfig.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatixBGStatsConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "BGStatsConfig") == 0)
    {
        __result = BGStatsConfig.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatixBGStatsConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "BGStatsConfig") == 0)
    {
        __result = BGStatsConfig.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatixBGStatsConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // BGStatsConfig
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("BGStatsConfig", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "BGStatsConfig") == 0)
    {
        __result = BGStatsConfig.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool initGlobalEnvConfType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "depthMapEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)depthMapEnable, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool initGlobalEnvConfType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "depthMapEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            depthMapEnable = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }

    return __result;
}

bool initGlobalEnvConfType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // depthMapEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("depthMapEnable", ParameterParserDataType::Int32, (double)depthMapEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "depthMapEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)depthMapEnable, 0));
    }

    return __result;
}

bool initBGStatsConfType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "BGStatMode") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)BGStatMode, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool initBGStatsConfType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "BGStatMode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            BGStatMode = (initBGStatsModeType)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool initBGStatsConfType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // BGStatMode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("BGStatMode", ParameterParserDataType::Enum, (double)(UINT32)BGStatMode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "BGStatMode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)BGStatMode, 0));
    }

    return __result;
}

bool initDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "initialGain") == 0)
    {
        __result = initialGain.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "initialCCT") == 0)
    {
        ParameterUtils::DoubleToString((double)initialCCT, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "BGStatConf") == 0)
    {
        __result = BGStatConf.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "globalEnvConf") == 0)
    {
        __result = globalEnvConf.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool initDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbrglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "initialGain") == 0)
    {
        __result = initialGain.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "initialCCT") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            initialCCT = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "BGStatConf") == 0)
    {
        __result = BGStatConf.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "globalEnvConf") == 0)
    {
        __result = globalEnvConf.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool initDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // initialGain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("initialGain", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "initialGain") == 0)
    {
        __result = initialGain.GetInfo(&__path[__pos], __info);
    }

    // initialCCT
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("initialCCT", ParameterParserDataType::Float, (double)initialCCT));
    }
    else if (PARAMETER_STRCMP(__name, "initialCCT") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)initialCCT));
    }

    // BGStatConf
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("BGStatConf", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "BGStatConf") == 0)
    {
        __result = BGStatConf.GetInfo(&__path[__pos], __info);
    }

    // globalEnvConf
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("globalEnvConf", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "globalEnvConf") == 0)
    {
        __result = globalEnvConf.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool dynamicEnablementType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "luxIndexThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)luxIndexThreshold, __value, __size);
        __result = true;
    }

    return __result;
}

bool dynamicEnablementType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "luxIndexThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            luxIndexThreshold = (float)dblValue; 
        }
    }

    return __result;
}

bool dynamicEnablementType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // luxIndexThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("luxIndexThreshold", ParameterParserDataType::Float, (double)luxIndexThreshold));
    }
    else if (PARAMETER_STRCMP(__name, "luxIndexThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)luxIndexThreshold));
    }

    return __result;
}

bool awbSubModuleType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "dynamicEnablement") == 0)
    {
        __result = dynamicEnablement.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "initData") == 0)
    {
        __result = initData.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "bgStatsConfig") == 0)
    {
        __result = bgStatsConfig.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "sensorCalibData") == 0)
    {
        __result = sensorCalibData.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "dfs") == 0)
    {
        __result = dfs.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "tmpConv") == 0)
    {
        __result = tmpConv.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "cctGainAdj") == 0)
    {
        __result = cctGainAdj.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "config2AInputCheckAwb") == 0)
    {
        __result = config2AInputCheckAwb.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwb") == 0)
    {
        __result = config2AOutputCheckAwb.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool awbSubModuleType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "dynamicEnablement") == 0)
    {
        __result = dynamicEnablement.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "initData") == 0)
    {
        __result = initData.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "bgStatsConfig") == 0)
    {
        __result = bgStatsConfig.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "sensorCalibData") == 0)
    {
        __result = sensorCalibData.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "dfs") == 0)
    {
        __result = dfs.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "tmpConv") == 0)
    {
        __result = tmpConv.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "cctGainAdj") == 0)
    {
        __result = cctGainAdj.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "config2AInputCheckAwb") == 0)
    {
        __result = config2AInputCheckAwb.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwb") == 0)
    {
        __result = config2AOutputCheckAwb.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool awbSubModuleType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // moduleID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("moduleID", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // dynamicEnablement
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dynamicEnablement", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dynamicEnablement") == 0)
    {
        __result = dynamicEnablement.GetInfo(&__path[__pos], __info);
    }

    // initData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("initData", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "initData") == 0)
    {
        __result = initData.GetInfo(&__path[__pos], __info);
    }

    // bgStatsConfig
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bgStatsConfig", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "bgStatsConfig") == 0)
    {
        __result = bgStatsConfig.GetInfo(&__path[__pos], __info);
    }

    // sensorCalibData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("sensorCalibData", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "sensorCalibData") == 0)
    {
        __result = sensorCalibData.GetInfo(&__path[__pos], __info);
    }

    // dfs
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dfs", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dfs") == 0)
    {
        __result = dfs.GetInfo(&__path[__pos], __info);
    }

    // tmpConv
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tmpConv", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "tmpConv") == 0)
    {
        __result = tmpConv.GetInfo(&__path[__pos], __info);
    }

    // cctGainAdj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cctGainAdj", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "cctGainAdj") == 0)
    {
        __result = cctGainAdj.GetInfo(&__path[__pos], __info);
    }

    // config2AInputCheckAwb
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("config2AInputCheckAwb", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "config2AInputCheckAwb") == 0)
    {
        __result = config2AInputCheckAwb.GetInfo(&__path[__pos], __info);
    }

    // config2AOutputCheckAwb
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("config2AOutputCheckAwb", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "config2AOutputCheckAwb") == 0)
    {
        __result = config2AOutputCheckAwb.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool awbSubModuleType::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool awbSubModuleType::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool awbSubModuleType::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_BGStatsROIConfigType(
    ParameterFileSymbolTableEntry* const entry,
    BGStatsROIConfigType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(BGStatsROIConfigType::ROIConfigTypeCroppedFOV):
            enumValue = BGStatsROIConfigType::ROIConfigTypeCroppedFOV;
            result    = true;
            break;
        case static_cast<UINT32>(BGStatsROIConfigType::ROIConfigTypeFullFOV):
            enumValue = BGStatsROIConfigType::ROIConfigTypeFullFOV;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to BGStatsROIConfigType", uintValue);
            break;
        }
    }
    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_initBGStatsModeType(
    ParameterFileSymbolTableEntry* const entry,
    initBGStatsModeType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(initBGStatsModeType::ZoomFOV):
            enumValue = initBGStatsModeType::ZoomFOV;
            result    = true;
            break;
        case static_cast<UINT32>(initBGStatsModeType::FullFOV):
            enumValue = initBGStatsModeType::FullFOV;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to initBGStatsModeType", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
awbSubModuleTypeClass::awbSubModuleTypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(dynamicEnablement);
    PARAMETER_INITIALIZE(initData);
    PARAMETER_INITIALIZE(bgStatsConfig);
    PARAMETER_INITIALIZE(sensorCalibData);
    PARAMETER_INITIALIZE(dfs);
    PARAMETER_INITIALIZE(tmpConv);
    PARAMETER_INITIALIZE(cctGainAdj);
    PARAMETER_INITIALIZE(config2AInputCheckAwb);
    PARAMETER_INITIALIZE(config2AOutputCheckAwb);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
awbSubModuleTypeClass::awbSubModuleTypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(dynamicEnablement);
    PARAMETER_INITIALIZE(initData);
    PARAMETER_INITIALIZE(bgStatsConfig);
    PARAMETER_INITIALIZE(sensorCalibData);
    PARAMETER_INITIALIZE(dfs);
    PARAMETER_INITIALIZE(tmpConv);
    PARAMETER_INITIALIZE(cctGainAdj);
    PARAMETER_INITIALIZE(config2AInputCheckAwb);
    PARAMETER_INITIALIZE(config2AOutputCheckAwb);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
awbSubModuleTypeClass::~awbSubModuleTypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* awbSubModuleTypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "awbrsubmodule")) == 0)
    {
        name = "awbrsubmodule";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load config2AOutputCheckAwbType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool config2AOutputCheckAwbType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(config2AOutputCheckAwbGains);
    PARAMETER_INITIALIZE(config2AOutputCheckAwbCCT);

    result = config2AOutputCheckAwbGains.Load(entry, alignment);
    if (result)
    {
        result = config2AOutputCheckAwbCCT.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load config2AOutputCheckAwbType_config2AOutputCheckAwbGainsStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool config2AOutputCheckAwbType::config2AOutputCheckAwbGainsStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(r_threshold_start);
    PARAMETER_INITIALIZE(r_threshold_end);
    PARAMETER_INITIALIZE(g_threshold_start);
    PARAMETER_INITIALIZE(g_threshold_end);
    PARAMETER_INITIALIZE(b_threshold_start);
    PARAMETER_INITIALIZE(b_threshold_end);

    SYMBOL_TABLE_ENTRY_READ(entry, result, r_threshold_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, r_threshold_end, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, g_threshold_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, g_threshold_end, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, b_threshold_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, b_threshold_end, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load config2AOutputCheckAwbType_config2AOutputCheckAwbCCTStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool config2AOutputCheckAwbType::config2AOutputCheckAwbCCTStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(cct_threshold_start);
    PARAMETER_INITIALIZE(cct_threshold_end);

    SYMBOL_TABLE_ENTRY_READ(entry, result, cct_threshold_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, cct_threshold_end, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load config2AInputCheckAwbType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool config2AInputCheckAwbType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(config2AInputCheckValidStatsRatio);

    SYMBOL_TABLE_ENTRY_READ(entry, result, config2AInputCheckValidStatsRatio, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load cctGainAdjType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool cctGainAdjType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(adjFactors);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    if (result)
    {
        result = adjFactors.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmpConvType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmpConvType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(camStartConvSpeed);
    PARAMETER_INITIALIZE(camStartFrameCnt);
    PARAMETER_INITIALIZE(normalConvSpeed);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, camStartConvSpeed, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, camStartFrameCnt, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, normalConvSpeed, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load dfsType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool dfsType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(dfsTrigger);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)

    // Reading Array dfsTrigger from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        dfsTriggerCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, dfsTriggerCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            dfsTriggerID = pointerEntry->ID;
            DupObjInfo* dfsTriggerDupInfo = pointerEntry->GetDupObject(dfsTriggerCount, file, line);
            if (dfsTriggerDupInfo != nullptr && dfsTriggerDupInfo->Object != nullptr)
            {
                dfsTrigger = (awbrglobalelements::AWBTriggerFloatType*)dfsTriggerDupInfo->Object;
                if (dfsTrigger == nullptr)
                {
                    if (dfsTriggerDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "dfsTrigger duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            dfsTriggerDupInfo->File, dfsTriggerDupInfo->Line, dfsTriggerDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "dfsTrigger duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            dfsTriggerDupInfo->Line, dfsTriggerDupInfo->RefCount);
                    }
                }
            }

            if (dfsTrigger == nullptr)
            {
                if (result && dfsTriggerCount > 0)
                {
                    dfsTrigger = PARAMETER_NEW awbrglobalelements::AWBTriggerFloatType[dfsTriggerCount];
                    (void)pointerEntry->SetDupObject(dfsTrigger, dfsTriggerCount, file, line);
                }
                if (!result || dfsTrigger == nullptr)
                {
                    if (dfsTriggerCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate dfsTrigger\n");
                    }
                    dfsTriggerCount = 0;
                }
                for (UINT32 i = 0; result && i < dfsTriggerCount; i++)
                {
                    result = dfsTrigger[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (dfsTrigger == nullptr)
        {
            dfsTriggerCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load sensorCalibrationType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool sensorCalibrationType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(numOfCalFactors);
    PARAMETER_INITIALIZE(cctOfCalFactors);
    PARAMETER_INITIALIZE(goldenModuleCal);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, numOfCalFactors, alignment)
    if (result)
    {
        result = entry->ReadArray(cctOfCalFactors, 6, alignment);
    }
    for (UINT32 i = 0; result && i < 15; i++)
    {
        if (result)
        {
            result = goldenModuleCal[i].Load(entry, alignment);
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load blackleveloffsetType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool blackleveloffsetType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(OffsetSubtraction);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    if (result)
    {
        result = entry->ReadArray(OffsetSubtraction, 3, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load inverseWBConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool inverseWBConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(InverseWBGain);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    if (result)
    {
        result = entry->ReadArray(InverseWBGain, 3, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load bitDepthConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool bitDepthConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(bitDepth);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, bitDepth, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load BGStatsConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool BGStatsConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(numOfRegionsHorizontal);
    PARAMETER_INITIALIZE(numOfRegionsVertical);
    PARAMETER_INITIALIZE(enableStatsQuadSync);
    PARAMETER_INITIALIZE(enableSaturatedStats);
    PARAMETER_INITIALIZE(enableYComputedStats);
    PARAMETER_INITIALIZE(YComputationColorWeight);
    PARAMETER_INITIALIZE(minStatsChannelThreshold);
    PARAMETER_INITIALIZE(maxStatsChannelThreshold);
    PARAMETER_INITIALIZE(HWStatsSkipFactor);
    PARAMETER_INITIALIZE(ROIConfigControl);
    PARAMETER_INITIALIZE(bitDepthConfig);
    PARAMETER_INITIALIZE(InverseWBConfig);
    PARAMETER_INITIALIZE(BlackLevelOffsetConfig);

    SYMBOL_TABLE_ENTRY_READ(entry, result, numOfRegionsHorizontal, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, numOfRegionsVertical, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableStatsQuadSync, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableSaturatedStats, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableYComputedStats, alignment)
    if (result)
    {
        result = entry->ReadArray(YComputationColorWeight, 3, alignment);
    }
    if (result)
    {
        result = entry->ReadArray(minStatsChannelThreshold, 4, alignment);
    }
    if (result)
    {
        result = entry->ReadArray(maxStatsChannelThreshold, 4, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, HWStatsSkipFactor, alignment)
    if (result)
    {
        result = awbrsubmodule::ReadEnum_BGStatsROIConfigType(entry, ROIConfigControl, alignment);
    }
    if (result)
    {
        result = bitDepthConfig.Load(entry, alignment);
    }
    if (result)
    {
        result = InverseWBConfig.Load(entry, alignment);
    }
    if (result)
    {
        result = BlackLevelOffsetConfig.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatixBGStatsConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixBGStatsConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(BGStatsConfig);

    result = BGStatsConfig.Load(entry, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load initGlobalEnvConfType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool initGlobalEnvConfType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(depthMapEnable);

    SYMBOL_TABLE_ENTRY_READ(entry, result, depthMapEnable, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load initBGStatsConfType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool initBGStatsConfType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(BGStatMode);

    result = awbrsubmodule::ReadEnum_initBGStatsModeType(entry, BGStatMode, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load initDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool initDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(initialGain);
    PARAMETER_INITIALIZE(initialCCT);
    PARAMETER_INITIALIZE(BGStatConf);
    PARAMETER_INITIALIZE(globalEnvConf);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    if (result)
    {
        result = initialGain.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, initialCCT, alignment)
    if (result)
    {
        result = BGStatConf.Load(entry, alignment);
    }
    if (result)
    {
        result = globalEnvConf.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load dynamicEnablementType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool dynamicEnablementType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(luxIndexThreshold);

    SYMBOL_TABLE_ENTRY_READ(entry, result, luxIndexThreshold, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load awbSubModuleType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool awbSubModuleType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(dynamicEnablement);
    PARAMETER_INITIALIZE(initData);
    PARAMETER_INITIALIZE(bgStatsConfig);
    PARAMETER_INITIALIZE(sensorCalibData);
    PARAMETER_INITIALIZE(dfs);
    PARAMETER_INITIALIZE(tmpConv);
    PARAMETER_INITIALIZE(cctGainAdj);
    PARAMETER_INITIALIZE(config2AInputCheckAwb);
    PARAMETER_INITIALIZE(config2AOutputCheckAwb);

    result = moduleID.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    if (result)
    {
        result = dynamicEnablement.Load(entry, alignment);
    }
    if (result)
    {
        result = initData.Load(entry, alignment);
    }
    if (result)
    {
        result = bgStatsConfig.Load(entry, alignment);
    }
    if (result)
    {
        result = sensorCalibData.Load(entry, alignment);
    }
    if (result)
    {
        result = dfs.Load(entry, alignment);
    }
    if (result)
    {
        result = tmpConv.Load(entry, alignment);
    }
    if (result)
    {
        result = cctGainAdj.Load(entry, alignment);
    }
    if (result)
    {
        result = config2AInputCheckAwb.Load(entry, alignment);
    }
    if (result)
    {
        result = config2AOutputCheckAwb.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool awbSubModuleType::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload cctGainAdjType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void cctGainAdjType::Unload(ParameterFileSymbolTable* const __table)
{
    adjFactors.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload dfsType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void dfsType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < dfsTriggerCount; i++)
    {
        dfsTrigger[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(dfsTrigger))
    {
        PARAMETER_DELETE[] dfsTrigger;
    }
    dfsTrigger = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload awbSubModuleType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void awbSubModuleType::Unload(ParameterFileSymbolTable* const __table)
{
    dfs.Unload(__table);
    cctGainAdj.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void awbSubModuleType::Release()
{
    ((awbSubModuleTypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 awbSubModuleType::GetModified()
{
    return ((awbSubModuleTypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* awbSubModuleTypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    awbSubModuleTypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (awbSubModuleTypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW awbSubModuleTypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create awbSubModuleTypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create awbSubModuleTypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool awbSubModuleTypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        awbSubModuleTypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "awbSubModuleTypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void awbSubModuleTypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void awbSubModuleTypeClass::Release()
{
    ParameterModule::Release();
}

AWBRSUBMODULE_NAMESPACE_END
